@@ -1,6 +1,6 @@
 ;;; amsthm.el --- Style hook for the AMS-LaTeX amsthm package.
 
-;; Copyright (C) 1997, 2013--2015 Free Software Foundation, Inc.
+;; Copyright (C) 1997, 2013--2015, 2018 Free Software Foundation, Inc.
 
 ;; Author: Carsten Dominik <dominik@strw.leidenuniv.nl>
 ;; Maintainer: auctex-devel@gnu.org
@@ -31,6 +31,11 @@
 
 ;;; Code:
 
+;; Silence the compiler:
+(declare-function font-latex-add-keywords
+		  "font-latex"
+		  (keywords class))
+
 (defvar LaTeX-amsthm-package-options nil
   "Package options for the amsthm package.")
 
@@ -40,25 +45,47 @@
 defined with \"\\newtheoremstyle\".")
 
 (defvar LaTeX-amsthm-fontdecl
-  (mapcar (lambda (elt) (concat TeX-esc elt))
-	  '("itshape" "bfseries" "scshape"
-	    "ttfamily" "upshape" "mdseries"
-	    "rmfamily" "sffamily" "slshape"))
+  '(;; family
+    "rmfamily" "sffamily" "ttfamily"
+    ;; series
+    "mdseries" "bfseries"
+    ;; shape
+    "upshape" "itshape" "slshape" "scshape"
+    ;; size
+    "tiny"  "scriptsize" "footnotesize"
+    "small" "normalsize" "large"
+    "Large" "LARGE" "huge" "Huge"
+    ;; reset macro
+    "normalfont")
   "List of font declaration commands for \"\\newtheoremstyle\".")
 
+(defun LaTeX-arg-amsthm-fontdecl (optional &optional prompt)
+  "Prompt for font declaration commands in \"\\newtheoremstyle\".
+If OPTIONAL is non-nil, insert the resulting value as an optional
+argument.  Use PROMPT as the prompt string."
+  ;; `INITIAL-INPUT' (5th argument to `TeX-completing-read-multiple')
+  ;; is hard-coded to `TeX-esc'.
+  (let* ((crm-separator (regexp-quote TeX-esc))
+	 (fontdecl (mapconcat 'identity
+			      (TeX-completing-read-multiple
+			       (TeX-argument-prompt optional prompt "Font")
+			       LaTeX-amsthm-fontdecl nil nil TeX-esc)
+			      TeX-esc)))
+    (TeX-argument-insert fontdecl optional)))
+
 (defun LaTeX-amsthm-env-label (environment)
   "Insert ENVIRONMENT, query for an optional argument and prompt
 for label.  AUCTeX users should add ENVIRONMENT to
 `LaTeX-label-alist' via customize or in init-file with:
 
-  (add-to-list 'LaTeX-label-alist '(\"lemma\" . \"lem:\"))
+  (add-to-list \\='LaTeX-label-alist \\='(\"lemma\" . \"lem:\"))
 
 RefTeX users should customize or add ENVIRONMENT to
 `LaTeX-label-alist' and `reftex-label-alist', e.g.
 
-  (add-to-list 'LaTeX-label-alist '(\"lemma\" . \"lem:\"))
-  (add-to-list 'reftex-label-alist
-	       '(\"lemma\" ?m \"lem:\" \"~\\ref{%s}\"
+  (add-to-list \\='LaTeX-label-alist \\='(\"lemma\" . \"lem:\"))
+  (add-to-list \\='reftex-label-alist
+	       \\='(\"lemma\" ?m \"lem:\" \"~\\ref{%s}\"
 		 nil (\"Lemma\" \"lemma\") nil))"
   (let ((opthead (TeX-read-string
 		  (TeX-argument-prompt t nil "Heading"))))
@@ -97,7 +124,7 @@ make them available as new environments.  Update
 		 newthmstyle)))
 
 (add-hook 'TeX-auto-prepare-hook #'LaTeX-amsthm-auto-prepare t)
-(add-hook 'TeX-auto-prepare-hook #'LaTeX-amsthm-auto-cleanup t)
+(add-hook 'TeX-auto-cleanup-hook #'LaTeX-amsthm-auto-cleanup t)
 (add-hook 'TeX-update-style-hook #'TeX-auto-parse t)
 
 (TeX-add-style-hook
@@ -106,6 +133,23 @@ make them available as new environments.  Update
    (LaTeX-add-environments
     '("proof" LaTeX-amsthm-env-label))
    (TeX-add-symbols
+    ;; Overrule the defintion in `latex.el':
+    '("newtheorem"
+      (TeX-arg-eval
+       (lambda ()
+	 (let ((nthm (TeX-read-string
+		      (TeX-argument-prompt nil nil "Environment"))))
+	   (LaTeX-add-amsthm-newtheorems nthm)
+	   (LaTeX-add-environments (list nthm 'LaTeX-amsthm-env-label))
+	   (format "%s" nthm))))
+      [ TeX-arg-environment "Numbered like" ]
+      t [ (TeX-arg-eval progn (if (eq (save-excursion
+					(backward-char 2)
+					(preceding-char)) ?\])
+				  ()
+				(TeX-arg-counter t "Within counter"))
+			"") ])
+
     '("newtheorem*"
       (TeX-arg-eval
        (lambda ()
@@ -119,8 +163,7 @@ make them available as new environments.  Update
 	   (format "%s" heading)))))
 
     '("theoremstyle"
-      (TeX-arg-eval completing-read
-		    "Style: "
+      (TeX-arg-eval completing-read "Style: "
 		    LaTeX-amsthm-theoremstyle-list))
     "qedhere"
     "swapnumbers"
@@ -136,11 +179,9 @@ make them available as new environments.  Update
 	   (format "%s" nthmstyle))))
       (TeX-arg-length "Space above")
       (TeX-arg-length "Space below")
-      (TeX-arg-eval completing-read
-		    "Body font: " LaTeX-amsthm-fontdecl)
+      (LaTeX-arg-amsthm-fontdecl "Body font")
       "Indent amount"
-      (TeX-arg-eval completing-read
-		    "Theorem head font: " LaTeX-amsthm-fontdecl)
+      (LaTeX-arg-amsthm-fontdecl "Theorem head font")
       "Punctuation after head"
       (TeX-arg-length "Space after head")
       "Theorem head spec"))
